// Divider.h | by Doug Manley
///////////////////////////////////////////////////////////
//		The Divider class is designed to allow for easily
//	resizable window panes (more or less HTML frames) within
// a main window.
///////////////////////////////////////////////////////////
#ifndef DIVIDER_H
#define DIVIDER_H
///////////////////////////////////////////////////////////
enum {
	DIV_NONE,
	DIV_VERTICAL,
	DIV_HORIZONTAL
};

enum {
	DIV_ONLY = 0,
	DIV_LEFT = 0, DIV_RIGHT = 1,
	DIV_TOP = 0, DIV_BOTTOM = 1
};

const HWND DIV_HWND_NULL = (HWND)0;

enum {
	DIV_ERROR_NOCHANGE = -100,
	DIV_ERROR_ALREADYDIVIDED,
	DIV_ERROR_NOTDIVIDED,
	DIV_ERROR_SIZE,
	DIV_ERROR_NOTFINAL,
	DIV_NOERROR = 0
};

const int DIVIDER_SIZE_DEFAULT = 4;
///////////////////////////////////////////////////////////
class Divider {
public:
	Divider() {
		div_type = DIV_NONE;
		div_half = NULL;
		SetRECT( 0,0, 0,0 );
		div_hwnd = 0;
		div_barsize = DIVIDER_SIZE_DEFAULT;
	}
	~Divider() {
		if( div_half != NULL ) {
			delete [] div_half;
		}
	}

	// Set the size of the rectangle:
	int SetRECT( RECT &rc ) {
		return( SetRECT( rc.left,rc.top, rc.right,rc.bottom ) );
	}
	int SetRECT( int x,int y, int w,int h ) {
		SetRect( &div_rect, x,y, w,h );
		// As long as it's a real window,
		// Simply resize it and show it:
		if( div_hwnd != (HWND)DIV_HWND_NULL ) {
			SetWindowPos( div_hwnd, NULL,
				div_rect.left,div_rect.top,
				div_rect.right,div_rect.bottom,
				SWP_NOZORDER|SWP_SHOWWINDOW
			);
		} else {
			// If it's split vertically, send the message to each part:
			if( div_type == DIV_VERTICAL ) {
				Divider *divTop = Divider::GetHalf(DIV_TOP);
				Divider *divBottom = Divider::GetHalf(DIV_BOTTOM);
				RECT rcTop = divTop->GetRECT();
				divTop->SetRECT(
					div_rect.left, div_rect.top,
					div_rect.right, rcTop.bottom // rcTop.bottom tells us the Length of the top part.
				);
				divBottom->SetRECT(
					div_rect.left, div_rect.top+rcTop.bottom+div_barsize,
					div_rect.right, div_rect.bottom-(rcTop.bottom+div_barsize)
				);
			} else if( div_type == DIV_HORIZONTAL ) {
				Divider *divLeft = Divider::GetHalf(DIV_LEFT);
				Divider *divRight = Divider::GetHalf(DIV_RIGHT);
				RECT rcLeft = divLeft->GetRECT();
				//RECT rcRight = divRight->GetRECT();
				divLeft->SetRECT(
					div_rect.left, div_rect.top,
					rcLeft.right, div_rect.bottom
				);
				divRight->SetRECT(
					rcLeft.right+div_barsize, div_rect.top,
					div_rect.right-(rcLeft.right+div_barsize), div_rect.bottom
				);
			}
		}
		return(DIV_NOERROR);
	}
	RECT GetRECT() {
		return( div_rect );
	}

	// Divide a divider into its parts.
	int Divide( int how, int length ) {
		if( how == DIV_NONE ) {
			if( div_half != NULL ) {
				div_type = how;
				delete [] div_half;
				div_half = NULL;
			} else {
				return(DIV_ERROR_NOCHANGE);
			}
		} else if( how == DIV_VERTICAL ) {
			if( div_half != NULL ) {
				return(DIV_ERROR_ALREADYDIVIDED);
			} else {
				if( length+div_barsize >= div_rect.bottom ) {
					return(DIV_ERROR_SIZE);
				} else {
					Divider::SetWindow(DIV_HWND_NULL);
					div_type = how;
					div_half = new Divider[2];
					div_half[DIV_TOP].SetRECT(
						div_rect.left, div_rect.top,
						div_rect.right, length
					);
					div_half[DIV_BOTTOM].SetRECT(
						div_rect.left, div_rect.top+length+div_barsize,
						div_rect.right, div_rect.bottom-length-div_barsize
					);
				}
			}
		} else if( how == DIV_HORIZONTAL ) {
			if( div_half != NULL ) {
				return(DIV_ERROR_ALREADYDIVIDED);
			} else {
				if( length+div_barsize >= div_rect.right ) {
					return(DIV_ERROR_SIZE);
				} else {
					Divider::SetWindow(DIV_HWND_NULL);
					div_type = how;
					div_half = new Divider[2];
					div_half[DIV_LEFT].SetRECT(
						div_rect.left, div_rect.top,
						length, div_rect.bottom
					);
					div_half[DIV_RIGHT].SetRECT(
						div_rect.left+length+div_barsize, div_rect.top,
						div_rect.right-length-div_barsize, div_rect.bottom
					);
				}
			}
		}
		return(DIV_NOERROR);
	}
	// Set the window for showing in the divider.
	// Resize it to fit.
	int SetWindow( HWND hwnd ) {
		// If this isn't a non-divided divider:
		if( div_half != NULL ) {  return(DIV_ERROR_NOTFINAL);  }

		// Otherwise:
		div_hwnd = hwnd;

		// As long as it's a real window:
		if( div_hwnd != (HWND)DIV_HWND_NULL ) {
			SetWindowPos( div_hwnd, NULL,
				div_rect.left,div_rect.top,
				div_rect.right,div_rect.bottom,
				SWP_NOZORDER|SWP_SHOWWINDOW
			);
		}
		return(DIV_NOERROR);
	}

	Divider* GetHalf( int half ) {
		if( div_half == NULL ) {
			return(NULL);
		} else {
			return( &div_half[half] );
		}
	}

	// Get the area that its dividing bar covers:
	RECT GetDividerBarRECT() {
		RECT rc;
		if( div_type == DIV_NONE ) {
			SetRect( &rc, 0,0, 0,0 );
		} else if( div_type == DIV_VERTICAL ) {
			RECT rcTop = Divider::GetHalf(DIV_TOP)->GetRECT();
			SetRect( &rc,
				rcTop.left, rcTop.top+rcTop.bottom,
				rcTop.right-rcTop.left, div_barsize
			);
		} else if( div_type == DIV_HORIZONTAL ) {
			RECT rcTop = Divider::GetHalf(DIV_LEFT)->GetRECT();
			SetRect( &rc,
				rcTop.left+rcTop.right, rcTop.top,
				div_barsize, rcTop.bottom-rcTop.top
			);
		}
		return( rc );
	}

	// Move the bar (resize the other windows):
	int MoveBar( int dist ) {
		if( div_type == DIV_NONE ) {
			return(DIV_ERROR_NOTDIVIDED);
		} else if( div_type == DIV_VERTICAL ) {
			Divider *divTop = Divider::GetHalf(DIV_TOP);
			Divider *divBottom = Divider::GetHalf(DIV_BOTTOM);
			// Try it out to see if it works, first:
			RECT rcTop = divTop->GetRECT();
			RECT rcBottom = divBottom->GetRECT();
				rcTop.bottom += dist;
				rcBottom.top += dist;
				rcBottom.bottom -= dist;
			// Check for problems:
			if( rcTop.top+rcTop.bottom < div_rect.top ||
				 rcBottom.top >= div_rect.top+div_rect.bottom ) {
				 //rcBottom.top >= div_rect.bottom ) {
				return(DIV_ERROR_SIZE);
			}
			// No problems? Then resize the windows:
			divTop->SetRECT(
				rcTop.left, rcTop.top,
				rcTop.right, rcTop.bottom
			);
			divBottom->SetRECT(
				rcBottom.left, rcBottom.top,
				rcBottom.right, rcBottom.bottom
			);
		} else if( div_type == DIV_HORIZONTAL ) {
			Divider *divLeft = Divider::GetHalf(DIV_LEFT);
			Divider *divRight = Divider::GetHalf(DIV_RIGHT);
			// Try it out to see if it works, first:
			RECT rcLeft = divLeft->GetRECT();
			RECT rcRight = divRight->GetRECT();
				rcLeft.right += dist;
				rcRight.left += dist;
				rcRight.right -= dist;
			// Check for problems:
			if( rcLeft.left+rcLeft.right < div_rect.left ||
				 rcRight.left >= div_rect.left+div_rect.right ) {
				return(DIV_ERROR_SIZE);
			}
			// No problems? Then resize the windows:
			divLeft->SetRECT(
				rcLeft.left, rcLeft.top,
				rcLeft.right, rcLeft.bottom
			);
			divRight->SetRECT(
				rcRight.left, rcRight.top,
				rcRight.right, rcRight.bottom
			);
		}
		return(DIV_NOERROR);
	}

	int SetWindowCursor( int mx, int my ) {
		if( div_half == NULL ) {
			return(-1);
		} else {
			RECT rc = GetDividerBarRECT();
			if( mx >= rc.left && mx < rc.left+rc.right &&
				 my >= rc.top && my < rc.top+rc.bottom ) {
				if( div_type == DIV_VERTICAL ) {
					SetCursor( cursor_resize_vertical );
					return(0);
				} else if( div_type == DIV_HORIZONTAL ) {
					SetCursor( cursor_resize_horizontal );
					return(0);
				}
				return(-1);
			} else {
				int result = div_half[0].SetWindowCursor(mx,my);
				if( result == 0 ) {
					return(0);
				}
				result = div_half[1].SetWindowCursor(mx,my);
				if( result == 0 ) {
					return(0);
				}
			}
			return(-1);
		}
	}
private:
	// How it's divided:
	int div_type;
	// Pointer to the dividers within this one:
	Divider *div_half;
	// The divider's window (this is DIV_HWND_NULL if it can't/doesn't have one):
	HWND div_hwnd;
	// It's window's (x,y) (width,height);
	RECT div_rect;
	// The size of the dividing bar:
	char div_barsize;
};
///////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////
#endif
///////////////////////////////////////////////////////////

